@@ -6,7 +6,7 @@
    fnasser@redhat.com    */
 
 /* Header file for GDB-specific command-line stuff.
-   Copyright (C) 1986-2013 Free Software Foundation, Inc.
+   Copyright (C) 1986-2016 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -55,10 +55,6 @@ extern struct cmd_list_element *detachlist;
 
 extern struct cmd_list_element *killlist;
 
-/* Chain containing all defined toggle subcommands.  */
-
-extern struct cmd_list_element *togglelist;
-
 /* Chain containing all defined stop subcommands.  */
 
 extern struct cmd_list_element *stoplist;
@@ -111,6 +107,10 @@ extern struct cmd_list_element *setprintlist;
 
 extern struct cmd_list_element *showprintlist;
 
+extern struct cmd_list_element *setprintrawlist;
+
+extern struct cmd_list_element *showprintrawlist;
+
 extern struct cmd_list_element *setprinttypelist;
 
 extern struct cmd_list_element *showprinttypelist;
