@@ -1,6 +1,6 @@
 /* This testcase is part of GDB, the GNU debugger.
 
-   Copyright 2011 Free Software Foundation, Inc.
+   Copyright 2011-2016 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,11 +25,21 @@ do_nothing (wchar_t *c)
 int
 main (void)
 {
+  int i;
   wchar_t narrow = 97;
   wchar_t single = 0xbeef;
   wchar_t simple[] = L"facile";
   wchar_t difficile[] = { 0xdead, 0xbeef, 0xfeed, 0xface};
   wchar_t mixed[] = {L'f', 0xdead, L'a', L'c', 0xfeed, 0xface};
+  wchar_t *cent = L"\242";
+  wchar_t repeat[128];
+  wchar_t *repeat_p = repeat;
+
+  repeat[0] = 0;
+  wcscat (repeat, L"A");
+  for (i = 0; i < 21; ++i)
+    wcscat (repeat, cent);
+  wcscat (repeat, L"B");
 
   do_nothing (&narrow); /* START */
   do_nothing (&single);
