@@ -1,11 +1,11 @@
 /* Interface definition for configurable Xtensa ISA support.
-   Copyright 2003, 2004, 2005 Free Software Foundation, Inc.
+   Copyright (C) 2003-2016 Free Software Foundation, Inc.
 
    This file is part of BFD, the Binary File Descriptor library.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -15,7 +15,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.  */
+   Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, 
+   USA.  */
 
 #ifndef XTENSA_LIBISA_H
 #define XTENSA_LIBISA_H
@@ -24,9 +25,6 @@
 extern "C" {
 #endif
 
-/* Use the statically-linked version for the GNU tools.  */
-#define STATIC_LIBISA 1
-
 /* Version number: This is intended to help support code that works with
    versions of this library from multiple Xtensa releases.  */
 
@@ -137,8 +135,8 @@ typedef int xtensa_funcUnit;
    retrieved with the "errno" function.  For any result other than
    xtensa_isa_ok, an error message containing additional information
    about the problem can be retrieved using the "error_msg" function.
-   The error messages are stored in an internal buffer, which should not
-   should be freed and may be overwritten by subsequent operations.  */
+   The error messages are stored in an internal buffer, which should
+   not be freed and may be overwritten by subsequent operations.  */
 
 typedef enum xtensa_isa_status_enum
 {
@@ -689,6 +687,13 @@ extern int
 xtensa_state_is_exported (xtensa_isa isa, xtensa_state st);
 
 
+/* Check for a "shared_or" state.  Returns 0 if the condition is false,
+   1 if the condition is true, and XTENSA_UNDEFINED on error.  */
+
+extern int
+xtensa_state_is_shared_or (xtensa_isa isa, xtensa_state st);
+
+
 
 /* Sysregs ("special registers" and "user registers").  */
 
