@@ -1,5 +1,5 @@
 /* sim-main.h -- Simulator for Motorola 68HC11 & 68HC12
-   Copyright (C) 1999-2015 Free Software Foundation, Inc.
+   Copyright (C) 1999-2016 Free Software Foundation, Inc.
    Written by Stephane Carrez (stcarrez@nerim.fr)
 
 This file is part of GDB, the GNU debugger.
@@ -20,14 +20,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 #ifndef _SIM_MAIN_H
 #define _SIM_MAIN_H
 
-#define WITH_MODULO_MEMORY 1
-#define WITH_WATCHPOINTS 1
-#define SIM_HANDLES_LMA 1
-
 #include "sim-basics.h"
-
-typedef address_word sim_cia;
-
 #include "sim-signal.h"
 #include "sim-base.h"
 
@@ -551,7 +544,7 @@ extern int cpu_initialize (SIM_DESC sd, sim_cpu *cpu);
 
 /* Returns the address of a 68HC12 indexed operand.
    Pre and post modifications are handled on the source register.  */
-extern uint16 cpu_get_indexed_operand_addr (sim_cpu* cpu, int restrict);
+extern uint16 cpu_get_indexed_operand_addr (sim_cpu *cpu, int restricted);
 
 extern void cpu_return (sim_cpu *cpu);
 extern void cpu_set_sp (sim_cpu *cpu, uint16 val);
@@ -572,23 +565,11 @@ extern void m68hc11cpu_set_port (struct hw *me, sim_cpu *cpu,
 
 /* The current state of the processor; registers, memory, etc.  */
 
-#define CIA_GET(CPU)      (cpu_get_pc (CPU))
-#define CIA_SET(CPU,VAL)  (cpu_set_pc ((CPU), (VAL)))
-
-#if (WITH_SMP)
-#define STATE_CPU(sd,n) (&(sd)->cpu[n])
-#else
-#define STATE_CPU(sd,n) (&(sd)->cpu[0])
-#endif
-
 struct sim_state {
-  sim_cpu        cpu[MAX_NR_PROCESSORS];
-  device         *devices;
+  sim_cpu        *cpu[MAX_NR_PROCESSORS];
   sim_state_base base;
 };
 
-extern void sim_set_profile (int n);
-extern void sim_set_profile_size (int n);
 extern void sim_board_reset (SIM_DESC sd);
 
 #define PRINT_TIME  0x01
